"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const q_1 = require("./q");
function getFields(info) {
    const fields = [];
    info.fieldNodes[0].selectionSet.selections.map((value, key) => {
        if (!value.selectionSet)
            fields.push(value.name.value);
    });
    return fields;
}
function getWheres(info) {
    const wheres = [];
    info.fieldNodes[0].arguments.map((val, key) => {
        const field = val.name.value;
        const value = val.value.value;
        if (field !== 'limit') {
            wheres.push({ field, value, operator: '=' });
        }
    });
    return wheres;
}
function getLimit(info) {
    let limit;
    info.fieldNodes[0].arguments.map((value, key) => {
        if (value.name.value === 'limit') {
            limit = value.value.value;
        }
    });
    return limit;
}
class Salesforce {
    constructor(props) {
        this.query = (parent, info) => {
            const queryBuilder = new q_1.default(info.returnType.ofType || info.returnType).select(getFields(info));
            const limit = getLimit(info);
            if (limit) {
                queryBuilder.limit(limit);
            }
            const wheres = getWheres(info);
            wheres.map(({ field, operator, value }) => queryBuilder.where(field, operator, value));
            Object.keys(parent).map((key) => queryBuilder.where(key, '=', parent[key]));
            const query = queryBuilder.build();
            console.log('\nExecuting', query);
            return this.conn.query(query);
        };
        this.conn = props.conn;
    }
}
exports.Salesforce = Salesforce;
