require("dotenv").config();
const express = require('express');
const bodyParser = require('body-parser');
const { graphqlExpress, graphiqlExpress } = require('apollo-server-express');
const { makeExecutableSchema } = require('graphql-tools');
const path = require('path');
const fs = require('fs');
const schemaFile = path.join(__dirname, 'schema.graphql');
const typeDefs = fs.readFileSync(schemaFile, 'utf8');
const jsforce = require('jsforce');

const conn = new jsforce.Connection({});

class DB {
  static conn;

  constructor() {
    this.conn
  }
}

conn.login(process.env.SALESFORCE_USERNAME, process.env.SALESFORCE_PASSWORD, function(err, userInfo) {
  if (err) console.error(err);
  const resolvers = {
    Query: {
      Accounts: (obj, args, context, info) =>
        conn.query(buildQuery(obj, args, context, info)).then(res => res.records),
      Account: (obj, args, context, info) =>
        conn.query(buildQuery(obj, args, context, info)).then(res => res.records[0]),
      Contacts: (obj, args, context, info) =>
        conn.query(buildQuery(obj, args, context, info)).then(res => res.records),
      Contact: (obj, args, context, info) =>
        conn.query(buildQuery(obj, args, context, info)).then(res => res.records[0]),
    },
    Contact: {
      Account: (obj, args, context, info) => {
        if (obj.AccountId)
          return conn
            .query(buildQuery({ Id: obj.AccountId }, args, context, info))
            .then(res => res.records[0]);
        else return null;
      },
    },
    Account: {
      Contacts: (obj, args, context, info) => {

        return conn
          .query(buildQuery({ AccountId: obj.Id}, args, context, info))
          .then(res => res.records)}
    },
  };

  // Put together a schema
  const schema = makeExecutableSchema({
    typeDefs,
    resolvers,
    context: req => ({
      ...req,
      db: 
    })
  });

  // Initialize the app
  const app = express();

  // The GraphQL endpoint
  app.use('/graphql', bodyParser.json(), graphqlExpress({ schema }));

  // GraphiQL, a visual editor for queries
  app.use('/graphiql', graphiqlExpress({ endpointURL: '/graphql' }));

  // Start the server
  app.listen(3000, () => {
    console.log('Go to http://localhost:3000/graphiql to run queries!');
  });
});
