"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsforce = require('jsforce');
require('dotenv').config();
const graphql_yoga_1 = require("graphql-yoga");
const express = require('express');
const bodyParser = require('body-parser');
const { graphqlExpress, graphiqlExpress } = require('apollo-server-express');
const { makeExecutableSchema } = require('graphql-tools');
const path = require('path');
const fs = require('fs');
const schemaFile = path.join(__dirname, 'schema.graphql');
const typeDefs = fs.readFileSync(schemaFile, 'utf8');
const { Salesforce } = require('../src');
const resolvers = {
    Query: {
        Accounts: (parent, args, context, info) => context.db.query({}, info).then(res => res.records),
        Account: (parent, args, context, info) => context.db.query({}, info).then(res => res.records[0]),
        Contacts: (parent, args, context, info) => context.db.query({}, info).then(res => res.records),
        Contact: (parentobj, args, context, info) => context.db.query({}, info).then(res => res.records[0]),
    },
};
const conn = new jsforce.Connection({});
conn.login(process.env.SALESFORCE_USERNAME, process.env.SALESFORCE_PASSWORD, (err, userinfo) => {
    init(conn);
});
function init(conn) {
    const client = new Salesforce({ conn });
    // Put together a schema
    const server = new graphql_yoga_1.GraphQLServer({
        typeDefs,
        resolvers,
        context: req => (Object.assign({}, req, { db: client })),
    });
    server.start({ playground: '/playground' }, ({ port }) => console.log('Server is running on localhost:' + port));
}
