require("dotenv").config();
const express = require('express');
const bodyParser = require('body-parser');
const { graphqlExpress, graphiqlExpress } = require('apollo-server-express');
const { makeExecutableSchema } = require('graphql-tools');
const path = require('path');
const fs = require('fs');
const schemaFile = path.join(__dirname, 'schema.graphql');
const typeDefs = fs.readFileSync(schemaFile, 'utf8');
const jsforce = require('jsforce');

const conn = new jsforce.Connection({
  // you can change loginUrl to connect to sandbox or prerelease env.
  // loginUrl : 'https://test.salesforce.com'
});
conn.login(process.env.SALESFORCE_USERNAME, process.env.SALESFORCE_PASSWORD, function(err, userInfo) {
  if (err) console.error(err);
  const resolvers = {
    Query: {
      Accounts: (obj, args, context, info) =>
        conn.query(buildQuery(obj, args, context, info)).then(res => res.records),
      Account: (obj, args, context, info) =>
        conn.query(buildQuery(obj, args, context, info)).then(res => res.records[0]),
      Contacts: (obj, args, context, info) =>
        conn.query(buildQuery(obj, args, context, info)).then(res => res.records),
      Contact: (obj, args, context, info) =>
        conn.query(buildQuery(obj, args, context, info)).then(res => res.records[0]),
    },
    Contact: {
      Account: (obj, args, context, info) => {
        if (obj.AccountId)
          return conn
            .query(buildQuery({ Id: obj.AccountId }, args, context, info))
            .then(res => res.records[0]);
        else return null;
      },
    },
    Account: {
      Contacts: (obj, args, context, info) => {

        return conn
          .query(buildQuery({ AccountId: obj.Id}, args, context, info))
          .then(res => res.records)}
    },
  };

  function buildQuery(obj, args, context, info) {
    const fields = [];
    info.fieldNodes[0].selectionSet.selections.map((value, key) => {
      if (!value.selectionSet) fields.push(value.name.value);
    });
    const wheres = [];

    Object.keys(args).map(key => {
      if (key !== 'limit') {
        wheres.push(`${key} = '${args[key]}'`);
      }
    });
    if (obj)
      Object.keys(obj).map(key => {
        if (key !== 'limit') {
          wheres.push(`${key} = '${obj[key]}'`);
        }
      });

    const query = `SELECT ${fields.join(',')} from ${
      info.returnType.ofType ? info.returnType.ofType : info.returnType
    } ${wheres.length != 0 ? ` WHERE ${wheres.join(' AND ')}` : ''} ${
      args.limit ? ` LIMIT ${args.limit}` : ''
    }`;
    console.log(query);
    return query;
  }

  // Put together a schema
  const schema = makeExecutableSchema({
    typeDefs,
    resolvers,
  });

  // Initialize the app
  const app = express();

  // The GraphQL endpoint
  app.use('/graphql', bodyParser.json(), graphqlExpress({ schema }));

  // GraphiQL, a visual editor for queries
  app.use('/graphiql', graphiqlExpress({ endpointURL: '/graphql' }));

  // Start the server
  app.listen(3000, () => {
    console.log('Go to http://localhost:3000/graphiql to run queries!');
  });
});
